'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toBlockStatement = toBlockStatement;
exports.toStatement = toStatement;
exports.toFunctionBody = toFunctionBody;
exports.ensureBlockBody = ensureBlockBody;
exports.blockToExpression = blockToExpression;
exports.getScopeParent = getScopeParent;

var _babelInstance = require('./babelInstance');

var _is = require('./is');

var _is2 = _interopRequireDefault(_is);

var _getLoc = require('ast-loc-utils/lib/getLoc');

var _getLoc2 = _interopRequireDefault(_getLoc);

var _buildAtLoc = require('ast-loc-utils/lib/buildAtLoc');

var _buildAtLoc2 = _interopRequireDefault(_buildAtLoc);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function toBlockStatement(body) {
  var t = (0, _babelInstance.getTypes)();
  var bodyLoc = (0, _getLoc2.default)(body);

  if ((0, _is.isa)(body, "BlockStatement")) return body;else if ((0, _is.isa)(body, "Statement")) return (0, _buildAtLoc2.default)(bodyLoc, t.blockStatement, [body]);else return (0, _buildAtLoc2.default)(bodyLoc, t.blockStatement, [(0, _buildAtLoc2.default)(bodyLoc, t.expressionStatement, body)]);
}

function toStatement(body) {
  var t = (0, _babelInstance.getTypes)();

  if ((0, _is.isa)(body, "Expression")) {
    return (0, _buildAtLoc2.default)((0, _getLoc2.default)(body), t.expressionStatement, body);
  } else return body;
}

function toFunctionBody(body) {
  if ((0, _is.isa)(body, "Expression")) return body;else return toBlockStatement(body);
}

function ensureBlockBody(path) {
  var t = (0, _babelInstance.getTypes)();
  if (!(0, _is2.default)("BlockStatement", path.node.body)) {
    return path.get("body").replaceWith((0, _buildAtLoc2.default)((0, _getLoc2.default)(path.node.body), t.blockStatement, [path.node.body]));
  }
}

function blockToExpression(path) {
  var t = (0, _babelInstance.getTypes)();
  var node = path.node;

  if ((0, _is.isa)(node, "BlockStatement")) {
    path.canSwapBetweenExpressionAndStatement = function () {
      return true;
    };
    // XXX: source maps - this code is inside babel and makes unmapped
    // nodes.
    path.replaceExpressionWithStatements(path.node.body);
    return path.node;
  } else if ((0, _is.isa)(node, "FunctionDeclaration")) {
    // Special case: the parser promotes a block consisting of a single declaration
    // from ExprStatement(FunctionExpr) to FunctionDeclaration. Convert back to
    // expression here.
    var nextNode = t.clone(node);
    if (nextNode.type === "NamedArrowDeclaration") {
      nextNode.type = "NamedArrowExpression";
    } else {
      nextNode.type = "FunctionExpression";
    }return nextNode;
  } else if ((0, _is.isa)(node, "ExpressionStatement")) {
    return node.expression;
  } else {
    return node;
  }
}

function getScopeParent(path) {
  return path.findParent(function (path) {
    return path.isFunction() || path.isBlockStatement() || path.isProgram();
  });
}