'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformForInArrayStatement = transformForInArrayStatement;
exports.transformForInObjectStatement = transformForInObjectStatement;

var _babelInstance = require('./babelInstance');

var _blocks = require('./blocks');

var _is = require('./is');

var _is2 = _interopRequireDefault(_is);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function generateForInIterator(path, type) {
  var t = (0, _babelInstance.getTypes)();
  var idx = path.node.idx || path.scope.generateUidIdentifier("i");

  // _i = 0
  var initDeclarations = [t.variableDeclarator(idx, t.numericLiteral(0))];

  // (_obj|_arr) = <expr>
  var refId = (0, _is2.default)("Identifier", path.node[type]) ? path.node[type] : function () {
    // if the target of iteration is a complex expression,
    // create a reference so it only evaluates once
    var refName = type === "object" ? "obj" : "arr";
    var refId = path.scope.generateUidIdentifier(refName);
    initDeclarations.unshift(t.variableDeclarator(refId, path.node[type]));
    return refId;
  }();

  // _keys = Object.keys(_obj)
  var keys = void 0;
  if (type === "object") {
    keys = path.scope.generateUidIdentifier("keys");
    initDeclarations.push(t.variableDeclarator(keys, t.callExpression(t.memberExpression(t.identifier("Object"), t.identifier("keys")), [refId])));
  }

  // _len = (_keys | _arr).length
  var len = path.scope.generateUidIdentifier("len");
  initDeclarations.push(t.variableDeclarator(len, t.memberExpression(type === "object" ? keys : refId, t.identifier("length"))));

  var init = t.variableDeclaration("let", initDeclarations);
  // _i < _len
  var test = t.binaryExpression("<", idx, len);
  // _i++
  var update = t.updateExpression("++", idx);

  (0, _blocks.ensureBlockBody)(path);
  var innerDeclarations = [];
  if (type === "object") {
    // _k = _keys[_i]
    var key = path.node.key || path.scope.generateUidIdentifier("k");
    innerDeclarations.push(t.variableDeclaration("const", [t.variableDeclarator(key, t.memberExpression(keys, idx, true))]));

    // val = _obj[_k]
    if (path.node.val) {
      innerDeclarations.push(t.variableDeclaration("const", [t.variableDeclarator(path.node.val, t.memberExpression(refId, key, true))]));
    }
  } else {
    // elem = _arr[_i]
    if (path.node.elem) {
      innerDeclarations.push(t.variableDeclaration("const", [t.variableDeclarator(path.node.elem, t.memberExpression(refId, idx, true))]));
    }
  }

  if (innerDeclarations.length > 0) {
    path.get("body").unshiftContainer("body", innerDeclarations);
  }

  return t.forStatement(init, test, update, path.node.body);
}

function transformForInArrayStatement(path) {
  return path.replaceWith(generateForInIterator(path, "array"));
}

function transformForInObjectStatement(path) {
  return path.replaceWith(generateForInIterator(path, "object"));
}