'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.defaultImports = exports.lodashImports = exports.everyLodashMethod = undefined;
exports.initializeStdlib = initializeStdlib;
exports.markIdentifier = markIdentifier;
exports.addImportSpecifier = addImportSpecifier;
exports.insertImports = insertImports;

var _lightscriptTypes = require('./lightscriptTypes');

var _lightscriptTypes2 = _interopRequireDefault(_lightscriptTypes);

var _blocks = require('./blocks');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var everyLodashMethod = exports.everyLodashMethod = ["add", "after", "ary", "assign", "assignIn", "assignInWith", "assignWith", "at", "attempt", "before", "bind", "bindAll", "bindKey", "camelCase", "capitalize", "castArray", "ceil", "chain", "chunk", "clamp", "clone", "cloneDeep", "cloneDeepWith", "cloneWith", "compact", "concat", "cond", "conforms", "conformsTo", "constant", "countBy", "create", "curry", "curryRight", "debounce", "deburr", "defaultTo", "defaults", "defaultsDeep", "defer", "delay", "difference", "differenceBy", "differenceWith", "divide", "drop", "dropRight", "dropRightWhile", "dropWhile", "each", "eachRight", "endsWith", "entries", "entriesIn", "eq", "escape", "escapeRegExp", "every", "extend", "extendWith", "fill", "filter", "find", "findIndex", "findKey", "findLast", "findLastIndex", "findLastKey", "first", "flatMap", "flatMapDeep", "flatMapDepth", "flatten", "flattenDeep", "flattenDepth", "flip", "floor", "flow", "flowRight", "forEach", "forEachRight", "forIn", "forInRight", "forOwn", "forOwnRight", "fromPairs", "functions", "functionsIn", "get", "groupBy", "gt", "gte", "has", "hasIn", "head", "identity", "inRange", "includes", "indexOf", "initial", "intersection", "intersectionBy", "intersectionWith", "invert", "invertBy", "invoke", "invokeMap", "isArguments", "isArray", "isArrayBuffer", "isArrayLike", "isArrayLikeObject", "isBoolean", "isBuffer", "isDate", "isElement", "isEmpty", "isEqual", "isEqualWith", "isError", "isFinite", "isFunction", "isInteger", "isLength", "isMap", "isMatch", "isMatchWith", "isNaN", "isNative", "isNil", "isNull", "isNumber", "isObject", "isObjectLike", "isPlainObject", "isRegExp", "isSafeInteger", "isSet", "isString", "isSymbol", "isTypedArray", "isUndefined", "isWeakMap", "isWeakSet", "iteratee", "join", "kebabCase", "keyBy", "keys", "keysIn", "last", "lastIndexOf", "lowerCase", "lowerFirst", "lt", "lte", "map", "mapKeys", "mapValues", "matches", "matchesProperty", "max", "maxBy", "mean", "meanBy", "memoize", "merge", "mergeWith", "method", "methodOf", "min", "minBy", "mixin", "multiply", "negate", "noConflict", "noop", "now", "nth", "nthArg", "omit", "omitBy", "once", "orderBy", "over", "overArgs", "overEvery", "overSome", "pad", "padEnd", "padStart", "parseInt", "partial", "partialRight", "partition", "pick", "pickBy", "property", "propertyOf", "pull", "pullAll", "pullAllBy", "pullAllWith", "pullAt", "random", "range", "rangeRight", "rearg", "reduce", "reduceRight", "reject", "remove", "repeat", "replace", "rest", "result", "reverse", "round", "runInContext", "sample", "sampleSize", "set", "setWith", "shuffle", "size", "slice", "snakeCase", "some", "sortBy", "sortedIndex", "sortedIndexBy", "sortedIndexOf", "sortedLastIndex", "sortedLastIndexBy", "sortedLastIndexOf", "sortedUniq", "sortedUniqBy", "split", "spread", "startCase", "startsWith", "stubArray", "stubFalse", "stubObject", "stubString", "stubTrue", "subtract", "sum", "sumBy", "tail", "take", "takeRight", "takeRightWhile", "takeWhile", "tap", "template", "templateSettings", "throttle", "thru", "times", "toArray", "toFinite", "toInteger", "toLength", "toLower", "toNumber", "toPairs", "toPairsIn", "toPath", "toPlainObject", "toSafeInteger", "toString", "toUpper", "transform", "trim", "trimEnd", "trimStart", "truncate", "unary", "unescape", "union", "unionBy", "unionWith", "uniq", "uniqBy", "uniqWith", "uniqueId", "unset", "unzip", "unzipWith", "update", "updateWith", "upperCase", "upperFirst", "values", "valuesIn", "without", "words", "wrap", "xor", "xorBy", "xorWith", "zip", "zipObject", "zipObjectDeep", "zipWith"];

var lodashImports = exports.lodashImports = everyLodashMethod.reduce(function (obj, methodName) {
  obj[methodName] = "lodash";
  return obj;
}, {});

var defaultImports = exports.defaultImports = Object.assign({}, lodashImports);

function initializeStdlib(opts) {
  var symbols = typeof opts.stdlib === "object" ? Object.assign({}, opts.stdlib.lodash === false ? {} : lodashImports) : defaultImports;

  return {
    enabled: opts.stdlib !== false,
    symbols,
    imports: {},
    useRequire: opts.stdlib && opts.stdlib.require === true
  };
}

function markIdentifier(stdlib, path) {
  if (!stdlib.enabled) return;

  var node = path.node;

  if (stdlib.symbols[node.name]) {
    var sp = (0, _blocks.getScopeParent)(path);
    if ((sp == null ? null : sp.scope) && !sp.scope.hasBinding(node.name)) {
      return addImportSpecifier(stdlib, node.name);
    }
  }
}

function addImportSpecifier(stdlib, symbol) {
  var importPath = stdlib.symbols[symbol];
  if (!importPath) return;
  if (!stdlib.imports[importPath]) {
    stdlib.imports[importPath] = [];
  }if (stdlib.imports[importPath].indexOf(symbol) < 0) {
    return stdlib.imports[importPath].push(symbol);
  }
}

function insertImports(stdlib, path) {
  var imports = stdlib.imports,
      useRequire = stdlib.useRequire;

  var declarations = [];

  for (var importPath in imports) {
    var specifierNames = imports[importPath];

    if (useRequire) {
      // eg; `const { map, uniq } = require('lodash');`
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = specifierNames[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var specifierName = _step.value;

          var importIdentifier = _lightscriptTypes2.default.identifier(specifierName);

          var requireCall = _lightscriptTypes2.default.callExpression(_lightscriptTypes2.default.identifier("require"), [_lightscriptTypes2.default.stringLiteral(importPath + '/' + specifierName)]);
          var requireStmt = _lightscriptTypes2.default.variableDeclaration("const", [_lightscriptTypes2.default.variableDeclarator(importIdentifier, requireCall)]);

          declarations.push(requireStmt);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    } else {
      // eg; `import { map, uniq } from 'lodash';`
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = specifierNames[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var _specifierName = _step2.value;

          var _importIdentifier = _lightscriptTypes2.default.identifier(_specifierName);
          var importSpecifier = _lightscriptTypes2.default.importDefaultSpecifier(_importIdentifier);
          var importDeclaration = _lightscriptTypes2.default.importDeclaration([importSpecifier], _lightscriptTypes2.default.stringLiteral(importPath + '/' + _specifierName));
          declarations.push(importDeclaration);
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    }
  }
  return path.unshiftContainer("body", declarations);
}